﻿// CommandExchange.cs
//
// Copyright(C) 2009 ubisys technologies GmbH, Duesseldorf, Germany.
// All rights reserved.
//
// www.ubisys.de
// support@ubisys.de
//
// THIS SOURCE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//
// IN NO EVENT SHALL UBISYS TECHNOLOGIES BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOURCE CODE OR THE USE OR
// OTHER DEALINGS IN THE SOURCE CODE.
//
// Part of the ubisys(R) 13.56 MHz RFID USB stick demo application
//
// Provides an easy-to-use command exchange path with devices that implement
// an "AT-like" interface, such as the ubisys(R) 13.56 MHz RFID stick. It 
// allows to send a request, and wait for a response frame. It provides some 
// low-level protocol processing (splits response frame into lines, checks 
// for an "OK" in the last line, adds carriage-returns to requests and 
// removes them from responses, etc.)
//
// Notice that this implementation is meant for virutal USB devices, as it 
// employs kind of a frame processing and might not work as expected with 
// other devices

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;

namespace ubisys
{
    class CommandExchange
    {
        private SerialPort _port;
        string _receiveBuffer;

        public CommandExchange()
        {
            _port = new SerialPort();
        }

        public void Open(string portName)
        {
            _port.PortName = portName;
            _port.ReadTimeout = 100;
            _port.ReceivedBytesThreshold = 1;

            // Register a notification sink which processes "data received!" 
            // indications
            _port.DataReceived += new SerialDataReceivedEventHandler(OnDataReceived);
            _port.Open();
        }

        private void OnDataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            _receiveBuffer += _port.ReadExisting();
        }

        public void SendRequest(string request)
        {
            byte[] commands = System.Text.Encoding.ASCII.GetBytes(request + '\r');

            // Send the command... 
            _port.Write(commands, 0, commands.Count());
        }

        public bool GetResponse(out string[] lines)
        {
            const int attemptLimit = 3;
            int attemptCount = 0;

            // ... and wait for a response
            while ((_receiveBuffer == null) && (++attemptCount < attemptLimit))
                System.Threading.Thread.Sleep(100);

            if (IsDataAvailable())
            {
                lines = _receiveBuffer.Split('\n');
                Purge();

                if (lines.Last() == "")
                    Array.Resize(ref lines, lines.Count() - 1);

                // Remove trailing carriage returns
                for (int i = 0; i < lines.Count(); i++)
                    lines[i] = lines[i].TrimEnd('\r');

                return true;
            }
            else
            {
                lines = null;
                return false;
            }
        }

        public bool ExchangeRequest(string request, out string[] lines)
        {
            // Clear any previously received data
            _receiveBuffer = null;

            SendRequest(request);

            if (!GetResponse(out lines))
                return false;

            return lines.Last() == "OK";
        }

        public void Purge()
        {
            _receiveBuffer = null;
        }

        public bool IsDataAvailable()
        {
            return _receiveBuffer != null;
        }
    }
}
