﻿// Enumerator.cs
//
// Copyright(C) 2012 ubisys technologies GmbH, Duesseldorf, Germany.
// All rights reserved.
//
// www.ubisys.de
// support@ubisys.de
//
// THIS SOURCE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//
// IN NO EVENT SHALL UBISYS TECHNOLOGIES BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOURCE CODE OR THE USE OR
// OTHER DEALINGS IN THE SOURCE CODE.
//
// Part of the ubisys(R) 13.56 MHz RFID USB stick demo application
//
// Provides the Enumerator class, which makes use of the Windows Device 
// Management API (setupapi.dll) to create a list of device paths 
// corresponding to suitable devices (serial ports in the first place).
// The list is then reduced to those devices that are USB devices and match 
// a given VID/PID combination. This allows the application to unambiguously
// identify 13.56 MHz USB sticks currently attached to the computer

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Win32;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace ubisys
{
    class Enumerator
    {
        public static void Enumerate(out List<string> ports)
        {
            Guid serenum = new Guid("4D36E978-E325-11CE-BFC1-08002BE10318");

            IntPtr hDevInfo = WindowsDeviceManagement.SetupDiGetClassDevs(ref serenum,
                null, (IntPtr)null, WindowsDeviceManagement.DIGCF_PRESENT |
                WindowsDeviceManagement.DIGCF_DEVICEINTERFACE);

            // Clear result vector
            ports = new List<string>();

            if (hDevInfo == WindowsDeviceManagement.INVALID_HANDLE_VALUE)
                throw new Win32Exception(Marshal.GetLastWin32Error());

            bool result = true;

            for (UInt32 item = 0; result; item++)
            {
                WindowsDeviceManagement.SP_DEVICE_INTERFACE_DATA ifcData =
                    new WindowsDeviceManagement.SP_DEVICE_INTERFACE_DATA();

                ifcData.cbSize = (UInt32)Marshal.SizeOf(ifcData);

                result = WindowsDeviceManagement.SetupDiEnumDeviceInterfaces(hDevInfo,
                    IntPtr.Zero, ref serenum, item, ref ifcData) != 0 ? true : false;

                if (!result)
                {
                    result = (Marshal.GetLastWin32Error() == WindowsDeviceManagement.ERROR_NO_MORE_ITEMS);
                    break;
                }

                // Got a device. Get the details.
                UInt32 cbDetail = 0;

                // Determine required detail size
                result = result && !WindowsDeviceManagement.SetupDiGetDeviceInterfaceDetail(hDevInfo,
                    ref ifcData, IntPtr.Zero, 0, out cbDetail, IntPtr.Zero) &&
                    (Marshal.GetLastWin32Error() == WindowsDeviceManagement.ERROR_INSUFFICIENT_BUFFER);

                if (result)
                {
                    IntPtr pDetail = Marshal.AllocHGlobal((int)cbDetail);

                    try
                    {
						if (IntPtr.Size == 8) // for 64 bit operating systems 
							Marshal.WriteInt32(pDetail, 8);
						else // for 32 bit systems 
							Marshal.WriteInt32(pDetail,
								sizeof(UInt32) + Marshal.SystemDefaultCharSize);

                        WindowsDeviceManagement.SP_DEVINFO_DATA devinfo =
                            new WindowsDeviceManagement.SP_DEVINFO_DATA();

                        devinfo.cbSize = (UInt32)Marshal.SizeOf(devinfo);

                        result = result && WindowsDeviceManagement.SetupDiGetDeviceInterfaceDetail
                            (hDevInfo, ref ifcData, pDetail, cbDetail, out cbDetail, IntPtr.Zero);

                        if (result)
                        {
                            // Get the String containing the devicePathName.
                            string devicePath =
                                Marshal.PtrToStringAuto((IntPtr)((Int64)pDetail + sizeof(UInt32)));

                            ports.Add(devicePath);
                        }
                    }
                    finally
                    {
                        Marshal.FreeHGlobal(pDetail);
                    }
                }
            }

            WindowsDeviceManagement.SetupDiDestroyDeviceInfoList(hDevInfo);
        }


        public static bool GetUSBDeviceInformation(string devicePath, out UInt16 vendorID,
            out UInt16 productID, out string serialNumber)
        {
            // Extract serial number from fully qualified device path. A typical 
            // device path looks like this:
            // "\\?\usb#vid_19a6&pid_6124#0123456789ab#{a3de00ea-eb56-479c-91b2-977c7b6a4503}"
            string[] tokens = devicePath.Split('#');

            try
            {
                if (tokens.Count() != 4)
                    throw new ApplicationException();

                if (tokens[0] != "\\\\?\\usb")
                    throw new ApplicationException();

                // Vendor ID & Product ID
                if ((tokens[1].Length != 17) || (tokens[1].Substring(0, 4) != "vid_"))
                    throw new ApplicationException();

                vendorID = System.Convert.ToUInt16(tokens[1].Substring(4, 4), 16);

                if (tokens[1].Substring(8, 5) != "&pid_")
                    throw new ApplicationException();

                productID = System.Convert.ToUInt16(tokens[1].Substring(tokens[1].Length - 4, 4), 16);

                // Serial Number
                serialNumber = tokens[2];
            }
            catch (ApplicationException)
            {
                // out parameters must be assigned
                vendorID = 0xffff;
                productID = 0xffff;
                serialNumber = null;

                return false;
            }

            return true;
        }

        public static void SelectUSBDevices(ref List<string> ports, UInt16 vendorID,
            UInt16 productID)
        {
            // Walk the array of enumerated devices and filter out those that
            // are no USB devices or whose VID/PID do not match the requested
            // values
            for (int i = 0; i < ports.Count; )
            {
                UInt16 thisVendorID, thisProductID;
                string serialNumber;

                if (!GetUSBDeviceInformation(ports[i],
                    out thisVendorID, out thisProductID, out serialNumber) ||
                    ((vendorID != 0) && ((thisProductID != productID) ||
                    (thisVendorID != vendorID))))
                {
                    ports.RemoveAt(i);
                }
                else
                    i++;
            }
        }

        public static string GetPortNameFromDevicePath(string devicePath)
        {
            // Extract serial number from fully qualified device path. A typical 
            // device path looks like this:
            // "\\?\usb#vid_19a6&pid_6124#0123456789ab#{a3de00ea-eb56-479c-91b2-977c7b6a4503}"
            string[] tokens = devicePath.Split('#');

            if (tokens.Count() != 4)
                throw new ApplicationException();

            string key = "HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Enum\\USB\\" +
                tokens[1] + "\\" + tokens[2] + "\\Device Parameters";

            return Registry.GetValue(key, "PortName", "").ToString();
        }
    }
}
